th {
    align: center;
    text-align: center;
}
td {
    align: center;
    text-align: center;
}
.side_by_side_1 {
    display: inline-block;
}
.top_row {
    valign: center;
    align: center;
}

